""" Unit tests for core_linked_records_app.system.local_id.api.
"""

from unittest import TestCase
from unittest.mock import patch, MagicMock

from core_linked_records_app.components.local_id.models import LocalId
from core_linked_records_app.system.local_id import (
    api as local_id_system_api,
)
from core_main_app.commons import exceptions


class TestGetByName(TestCase):
    """Unit tests for `get_by_name` function."""

    def setUp(self) -> None:
        """setUp"""
        self.kwargs = {
            "record_name": "mock_name",
        }

    @patch.object(LocalId, "get_by_name")
    def test_local_id_get_by_name_called(self, mock_get_by_name):
        """test_local_id_get_by_name_called"""
        local_id_system_api.get_by_name(**self.kwargs)
        mock_get_by_name.assert_called_with(self.kwargs["record_name"])

    @patch.object(LocalId, "get_by_name")
    def test_get_by_name_does_not_exist_raises_does_not_exist(
        self, mock_get_by_name
    ):
        """test_get_by_name_does_not_exist_raises_does_not_exist"""
        mock_get_by_name.side_effect = exceptions.DoesNotExist(
            "mock_get_by_name_does_not_exist"
        )

        with self.assertRaises(exceptions.DoesNotExist):
            local_id_system_api.get_by_name(**self.kwargs)

    @patch.object(LocalId, "get_by_name")
    def test_get_by_name_exception_raises_api_error(self, mock_get_by_name):
        """test_get_by_name_exception_raises_api_error"""
        mock_get_by_name.side_effect = Exception("mock_get_by_name_exception")

        with self.assertRaises(exceptions.ApiError):
            local_id_system_api.get_by_name(**self.kwargs)

    @patch.object(LocalId, "get_by_name")
    def test_successful_execution_returns_local_id_get_by_name_value(
        self, mock_get_by_name
    ):
        """test_successful_execution_returns_local_id_get_by_name_value"""
        expected_value = MagicMock()
        mock_get_by_name.return_value = expected_value
        self.assertEqual(
            local_id_system_api.get_by_name(**self.kwargs), expected_value
        )


class TestGetByClassAndId(TestCase):
    """Unit tests for `get_by_class_and_id` function."""

    @patch.object(LocalId, "get_by_class_and_id")
    def test_get_by_class_and_id_does_not_exist_raises_does_not_exist(
        self, mock_get_by_class_and_id
    ):
        """test_get_by_class_and_id_failure_raises_api_error"""

        mock_get_by_class_and_id.side_effect = exceptions.DoesNotExist(
            "mock_get_by_class_and_id_exception"
        )

        with self.assertRaises(exceptions.DoesNotExist):
            local_id_system_api.get_by_class_and_id("class", "id")

    @patch.object(LocalId, "get_by_class_and_id")
    def test_get_by_class_and_id_failure_raises_api_error(
        self, mock_get_by_class_and_id
    ):
        """test_get_by_class_and_id_failure_raises_api_error"""

        mock_get_by_class_and_id.side_effect = Exception(
            "mock_get_by_class_and_id_exception"
        )

        with self.assertRaises(exceptions.ApiError):
            local_id_system_api.get_by_class_and_id("class", "id")

    @patch.object(LocalId, "get_by_class_and_id")
    def test_returns_get_by_class_and_id_output(
        self, mock_get_by_class_and_id
    ):
        """test_returns_get_by_class_and_id_output"""

        expected_result = "mock_get_by_class_and_id"
        mock_get_by_class_and_id.return_value = expected_result

        self.assertEqual(
            local_id_system_api.get_by_class_and_id("class", "id"),
            expected_result,
        )


class TestInsert(TestCase):
    """Test Insert"""

    def setUp(self) -> None:
        self.mock_local_id = LocalId(record_name="mock_record_name")

    @patch.object(LocalId, "upsert")
    def test_save_failure_raises_api_error(self, mock_save):
        """test_save_failure_raises_api_error"""

        mock_save.side_effect = Exception("mock_save_general_exception")

        with self.assertRaises(exceptions.ApiError):
            local_id_system_api.insert(self.mock_local_id)

    @patch.object(LocalId, "upsert")
    def test_returns_save_output(self, mock_save):
        """test_returns_save_output"""

        mock_save.return_value = None

        self.assertEqual(
            local_id_system_api.insert(self.mock_local_id), self.mock_local_id
        )


class TestDelete(TestCase):
    """Test Delete"""

    def setUp(self) -> None:
        self.mock_local_id = LocalId(record_name="mock_record_name")

    @patch.object(LocalId, "delete")
    def test_delete_raises_api_error(self, mock_delete):
        """test_delete_raises_api_error"""

        mock_delete.side_effect = Exception("mock_delete_exception")

        with self.assertRaises(exceptions.ApiError):
            local_id_system_api.delete(self.mock_local_id)

    @patch.object(LocalId, "delete")
    def test_returns_delete_output(self, mock_delete):
        """test_returns_delete_output"""

        expected_result = "mock_delete"
        mock_delete.return_value = expected_result

        self.assertEqual(
            local_id_system_api.delete(self.mock_local_id), expected_result
        )



""" ACL tests for `core_linked_records.components.oai_record.api`.
"""

from unittest import TestCase
from unittest.mock import MagicMock, patch

from core_linked_records_app.components.oai_record import api as oai_record_api
from core_linked_records_app.components.pid_path import api as pid_path_api
from core_main_app.access_control import api as main_acl_api
from core_main_app.access_control.exceptions import AccessControlError
from core_main_app.utils.tests_tools.MockUser import create_mock_user
from core_oaipmh_harvester_app.components.oai_record import (
    api as oaipmh_harvester_oai_record_api,
)


class TestGetPidForData(TestCase):
    """ACL tests for `get_pid_for_data` function."""

    def setUp(self) -> None:
        """setUp"""
        self.mock_request = MagicMock()
        self.mock_get_value_from_dot_notation_return_value = (
            "mock_get_value_from_dot_notation"
        )

    def setup_mocks(
        self,
        mock_get_by_id,
        mock_get_by_template,
        mock_get_value_from_dot_notation,
        user,
    ) -> None:
        """setup_mocks"""
        self.mock_request.user = user

        mock_get_by_id.return_value = MagicMock()
        mock_get_by_template.return_value = MagicMock()
        mock_get_value_from_dot_notation.return_value = (
            self.mock_get_value_from_dot_notation_return_value
        )

    @patch.object(oai_record_api, "get_value_from_dot_notation")
    @patch.object(pid_path_api, "get_by_template")
    @patch.object(oaipmh_harvester_oai_record_api, "get_by_id")
    def test_superuser_can_access(
        self,
        mock_get_by_id,
        mock_get_by_template,
        mock_get_value_from_dot_notation,
    ):
        """test_superuser_can_access"""
        user = create_mock_user("1", is_superuser=True)

        self.setup_mocks(
            mock_get_by_id,
            mock_get_by_template,
            mock_get_value_from_dot_notation,
            user,
        )

        self.assertEqual(
            oai_record_api.get_pid_for_data(
                "mock_oai_record_id", self.mock_request
            ),
            self.mock_get_value_from_dot_notation_return_value,
        )

    @patch.object(oai_record_api, "get_value_from_dot_notation")
    @patch.object(pid_path_api, "get_by_template")
    @patch.object(oaipmh_harvester_oai_record_api, "get_by_id")
    def test_registered_user_can_access(
        self,
        mock_get_by_id,
        mock_get_by_template,
        mock_get_value_from_dot_notation,
    ):
        """test_registered_user_can_access"""
        user = create_mock_user("1")

        self.setup_mocks(
            mock_get_by_id,
            mock_get_by_template,
            mock_get_value_from_dot_notation,
            user,
        )

        self.assertEqual(
            oai_record_api.get_pid_for_data(
                "mock_oai_record_id", self.mock_request
            ),
            self.mock_get_value_from_dot_notation_return_value,
        )

    @patch.object(oai_record_api, "get_value_from_dot_notation")
    @patch.object(pid_path_api, "get_by_template")
    @patch.object(oaipmh_harvester_oai_record_api, "get_by_id")
    @patch.object(main_acl_api, "settings")
    def test_anonymous_user_not_public_cannot_access(
        self,
        mock_settings,
        mock_get_by_id,
        mock_get_by_template,
        mock_get_value_from_dot_notation,
    ):
        """test_anonymous_user_not_public_cannot_access"""
        mock_settings.CAN_ANONYMOUS_ACCESS_PUBLIC_DOCUMENT = False
        user = create_mock_user("1", is_anonymous=True)

        self.setup_mocks(
            mock_get_by_id,
            mock_get_by_template,
            mock_get_value_from_dot_notation,
            user,
        )

        with self.assertRaises(AccessControlError):
            oai_record_api.get_pid_for_data(
                "mock_oai_record_id", self.mock_request
            )

    @patch.object(oai_record_api, "get_value_from_dot_notation")
    @patch.object(pid_path_api, "get_by_template")
    @patch.object(oaipmh_harvester_oai_record_api, "get_by_id")
    @patch.object(main_acl_api, "settings")
    def test_anonymous_user_and_public_can_access(
        self,
        mock_settings,
        mock_get_by_id,
        mock_get_by_template,
        mock_get_value_from_dot_notation,
    ):
        """test_anonymous_user_and_public_can_access"""
        mock_settings.CAN_ANONYMOUS_ACCESS_PUBLIC_DOCUMENT = True
        user = create_mock_user("1", is_anonymous=True)

        self.setup_mocks(
            mock_get_by_id,
            mock_get_by_template,
            mock_get_value_from_dot_notation,
            user,
        )

        self.assertEqual(
            oai_record_api.get_pid_for_data(
                "mock_oai_record_id", self.mock_request
            ),
            self.mock_get_value_from_dot_notation_return_value,
        )



""" Url router for the core linked records app
"""

from django.conf.urls import include
from django.urls import re_path

urlpatterns = [
    re_path(r"rest/", include("core_linked_records_app.rest.urls")),
]



""" Unit tests for core_linked_records_app.utils.dict
"""

from unittest import TestCase
from unittest.mock import patch

from django.core.exceptions import ValidationError

from core_linked_records_app.utils.dict import (
    get_value_from_dot_notation,
    is_dot_notation_in_dictionary,
    validate_dot_notation,
)
from core_main_app.commons.exceptions import QueryError


class TestValidateDict(TestCase):
    """Tests for validate_dict function"""

    @patch("core_linked_records_app.utils.dict.sanitize_value")
    def test_sanitize_value_query_error_raise_validation_error(
        self, mock_sanitize_value
    ):
        """test_sanitize_value_query_error_raise_validation_error"""
        mock_sanitize_value.side_effect = QueryError(
            "mock_sanitize_value_query_error"
        )

        with self.assertRaises(ValidationError):
            validate_dot_notation("mock_value")

    @patch("core_linked_records_app.utils.dict.sanitize_value")
    def test_sanitize_value_exception_raise_validation_error(
        self, mock_sanitize_value
    ):
        """test_sanitize_value_exception_raise_validation_error"""
        mock_sanitize_value.side_effect = Exception(
            "mock_sanitize_value_exception"
        )

        with self.assertRaises(ValidationError):
            validate_dot_notation("mock_value")

    @patch("core_linked_records_app.utils.dict.sanitize_value")
    def test_sanitize_value_success_returns_none(self, mock_sanitize_value):
        """test_sanitize_value_success_returns_none"""
        mock_sanitize_value.return_value = None

        self.assertIsNone(validate_dot_notation("mock_value"))


class TestGetValueFromDotNotation(TestCase):
    """Tests for get_value_from_dot_notation function"""

    @classmethod
    def setUpClass(cls) -> None:
        """Create test case global variables"""
        cls.mock_dictionary = {
            "root": {"elem11": {"elem21": "value2"}, "elem12": "value1"}
        }

    def test_empty_path_returns_dictionary(self):
        """Test empty path returns dictionary"""
        self.assertDictEqual(
            get_value_from_dot_notation(self.mock_dictionary, ""),
            self.mock_dictionary,
        )

    def test_non_existing_path_returns_none(self):
        """Test non existing path returns None"""
        self.assertIsNone(
            get_value_from_dot_notation(self.mock_dictionary, "root.elem13")
        )

    def test_existing_root_returns_dict_content(self):
        """Test existing root returns dict_content"""
        self.assertDictEqual(
            get_value_from_dot_notation(self.mock_dictionary, "root"),
            self.mock_dictionary["root"],
        )

    def test_existing_deep_key_returns_dict_content(self):
        """Test existing deep key returns dict_content"""
        self.assertEqual(
            get_value_from_dot_notation(
                self.mock_dictionary, "root.elem11.elem21"
            ),
            self.mock_dictionary["root"]["elem11"]["elem21"],
        )


class TestIsDotNotationInDictionary(TestCase):
    """Tests for is_dot_notation_in_dictionary function"""

    @classmethod
    def setUpClass(cls) -> None:
        """Create test case global variables"""
        cls.mock_dictionary = {
            "root": {"elem11": {"elem21": "value2"}, "elem12": "value1"}
        }

    def test_empty_path_returns_true(self):
        """Test empty path returns True"""
        self.assertTrue(
            is_dot_notation_in_dictionary(self.mock_dictionary, [])
        )

    def test_non_existing_path_returns_false(self):
        """Test non existing path returns False"""
        self.assertFalse(
            is_dot_notation_in_dictionary(self.mock_dictionary, "root.elem13")
        )

    def test_existing_root_key_returns_true(self):
        """Test existing root key returns True"""
        self.assertTrue(
            is_dot_notation_in_dictionary(self.mock_dictionary, "root")
        )

    def test_existing_deep_key_returns_true(self):
        """Test existing deep key returns True"""
        self.assertTrue(
            is_dot_notation_in_dictionary(
                self.mock_dictionary, "root.elem11.elem21"
            )
        )



""" Unit tests for core_linked_records_app.components.pid_path.api
"""

from unittest import TestCase
from unittest.mock import patch

from core_linked_records_app.components.pid_path import (
    api as pid_path_api,
)
from core_linked_records_app.components.pid_path.models import PidPath
from core_main_app.commons.exceptions import ApiError
from core_main_app.components.template import (
    api as template_api,
)
from core_main_app.utils.tests_tools.MockUser import create_mock_user
from tests import mocks


class TestGetByTemplateId(TestCase):
    """Test Get By Template Id"""

    def setUp(self) -> None:
        self.mock_user = create_mock_user("1")
        self.mock_template = mocks.MockDocument()
        self.mock_template.user = self.mock_user.id
        self.kwargs = {
            "template": self.mock_template,
            "user": self.mock_user,
        }

    @patch.object(PidPath, "get_by_template")
    def test_get_by_template_failure_raises_api_error(
        self, mock_get_by_template
    ):
        """test_get_by_template_failure_raises_api_error"""

        mock_get_by_template.side_effect = Exception(
            "mock_get_by_template_exception"
        )

        with self.assertRaises(ApiError):
            pid_path_api.get_by_template(**self.kwargs)

    @patch.object(PidPath, "__init__")
    @patch.object(PidPath, "get_by_template")
    def test_pid_path_init_failure_raises_api_error(
        self, mock_get_by_template, mock_pid_path
    ):
        """test_pid_path_init_failure_raises_api_error"""

        mock_get_by_template.return_value = None
        mock_pid_path.side_effect = Exception("mock_pid_path_exception")

        with self.assertRaises(ApiError):
            pid_path_api.get_by_template(**self.kwargs)

    @patch.object(PidPath, "__init__")
    @patch.object(PidPath, "get_by_template")
    def test_returns_new_pid_path_if_not_exists(
        self,
        mock_get_by_template,
        mock_pid_path,  # noqa, pylint: disable=unused-argument
    ):
        """test_returns_new_pid_path_if_not_exists"""

        expected_result = "mock_pid_path_object"
        mock_get_by_template.return_value = expected_result

        self.assertEqual(
            pid_path_api.get_by_template(**self.kwargs), expected_result
        )

    @patch.object(PidPath, "__new__")
    @patch.object(template_api, "get_by_id")
    @patch.object(PidPath, "get_by_template")
    def test_returns_pid_path_if_exists(
        self,
        mock_get_by_template,
        mock_get_by_id,
        mock_pid_path,
    ):
        """test_returns_pid_path_if_exists"""
        expected_result = "mock_pid_path"
        mock_get_by_template.return_value = None
        mock_get_by_id.return_value = self.mock_template
        mock_pid_path.return_value = expected_result

        self.assertEqual(
            pid_path_api.get_by_template(**self.kwargs), expected_result
        )


class TestGetAll(TestCase):
    """Test Get All"""

    def setUp(self) -> None:
        self.mock_request = mocks.MockRequest()
        self.mock_request.user = create_mock_user("1")

    @patch.object(template_api, "get_all")
    def test_template_api_get_all_failure_raises_api_error(self, mock_get_all):
        """test_template_api_get_all_failure_raises_api_error"""

        mock_get_all.side_effect = Exception("mock_get_all_exception")

        with self.assertRaises(ApiError):
            pid_path_api.get_all(self.mock_request)

    @patch.object(PidPath, "get_all_by_template_list")
    @patch.object(template_api, "get_all")
    def test_get_all_by_template_list_failure_raises_api_error(
        self, mock_get_all, mock_get_all_by_template_list
    ):
        """test_get_all_by_template_list_failure_raises_api_error"""

        mock_get_all.return_value = []
        mock_get_all_by_template_list.side_effect = Exception(
            "mock_get_all_exception"
        )

        with self.assertRaises(ApiError):
            pid_path_api.get_all(self.mock_request)

    @patch.object(PidPath, "get_all_by_template_list")
    @patch.object(template_api, "get_all")
    def test_returns_get_all_by_template_list_output(
        self, mock_get_all, mock_get_all_by_template_list
    ):
        """test_returns_get_all_by_template_list_output"""

        expected_result = "mock_get_all_by_template_list"
        mock_get_all.return_value = []
        mock_get_all_by_template_list.return_value = expected_result

        self.assertEqual(
            pid_path_api.get_all(self.mock_request), expected_result
        )



""" Permission tests for core_linked_records_app.rest.pid.views
"""

from unittest import TestCase
from unittest.mock import patch, Mock

from django.test import SimpleTestCase
from rest_framework import status

from core_explore_common_app.components.query import api as query_api
from core_linked_records_app.components.blob import api as blob_api
from core_linked_records_app.components.data import api as data_api
from core_linked_records_app.rest.pid import views as pid_views
from core_main_app.utils.tests_tools.MockUser import create_mock_user
from core_main_app.utils.tests_tools.RequestMock import RequestMock
from core_main_app.components.template_html_rendering import (
    api as template_html_rendering_api,
)
from tests import mocks


class TestRetrieveDataPidGet(TestCase):
    """Test Retrieve Data Pid Get"""

    @patch.object(data_api, "get_pid_for_data")
    def test_anonymous_returns_200(self, mock_get_pid_for_data):
        """test_anonymous_returns_200"""

        mock_get_pid_for_data.return_value = "mock_pid"

        response = RequestMock.do_request_get(
            pid_views.RetrieveDataPIDView.as_view(),
            None,
            data={"data_id": "mock_data_id"},
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)

    @patch.object(data_api, "get_pid_for_data")
    def test_authenticated_returns_200(self, mock_get_pid_for_data):
        """test_authenticated_returns_200"""

        mock_user = create_mock_user("1")

        mock_get_pid_for_data.return_value = "mock_pid"

        response = RequestMock.do_request_get(
            pid_views.RetrieveDataPIDView.as_view(),
            mock_user,
            data={"data_id": "mock_data_id"},
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)

    @patch.object(data_api, "get_pid_for_data")
    def test_staff_returns_200(self, mock_get_pid_for_data):
        """test_staff_returns_200"""

        mock_user = create_mock_user("1", is_staff=True)

        mock_get_pid_for_data.return_value = "mock_pid"

        response = RequestMock.do_request_get(
            pid_views.RetrieveDataPIDView.as_view(),
            mock_user,
            data={"data_id": "mock_data_id"},
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)


class TestRetrieveBlobPidGet(TestCase):
    """Test Retrieve Blob Pid Get"""

    @patch.object(blob_api, "get_pid_for_blob")
    def test_anonymous_returns_200(self, mock_get_pid_for_blob):
        """test_anonymous_returns_"""

        mock_get_pid_for_blob.return_value = mocks.MockLocalId()

        response = RequestMock.do_request_get(
            pid_views.RetrieveBlobPIDView.as_view(),
            None,
            data={"blob_id": "mock_blob_id"},
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)

    @patch.object(blob_api, "get_pid_for_blob")
    def test_authenticated_returns_200(self, mock_get_pid_for_blob):
        """test_authenticated_returns_200"""

        mock_user = create_mock_user("1")

        mock_get_pid_for_blob.return_value = mocks.MockLocalId()

        response = RequestMock.do_request_get(
            pid_views.RetrieveBlobPIDView.as_view(),
            mock_user,
            data={"blob_id": "mock_blob_id"},
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)

    @patch.object(blob_api, "get_pid_for_blob")
    def test_staff_returns_200(self, mock_get_pid_for_blob):
        """test_staff_returns_200"""

        mock_user = create_mock_user("1", is_staff=True)

        mock_get_pid_for_blob.return_value = mocks.MockLocalId()

        response = RequestMock.do_request_get(
            pid_views.RetrieveBlobPIDView.as_view(),
            mock_user,
            data={"blob_id": "mock_blob_id"},
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)


class TestRetrieveListPidPost(TestCase):
    """Test Retrieve List Pid Post"""

    def setUp(self) -> None:
        self.mock_data_source = dict(
            query_options={},
            order_by_field="",
            authentication=dict(
                auth_type="oauth2",
                params={"access_token": "mock_access_token"},
            ),
            capabilities={"query_pid": "mock_url_pid_list"},
        )

    def send_oauth_request(self, mock_query_get_by_id, mock_request, user):
        """send_post_request"""

        mock_query_get_by_id.return_value = mocks.MockQuery(
            data_sources=[self.mock_data_source]
        )
        mock_request.return_value = mocks.MockResponse(
            json_data={"pids": ["mock_pid"]}
        )

        return RequestMock.do_request_post(
            pid_views.RetrieveListPIDView.as_view(),
            user,
            data={"query_id": "mock_query_id", "data_source_index": "0"},
            content_type=None,
        )

    @patch.object(pid_views, "oauth2_post_request")
    @patch.object(query_api, "get_by_id")
    def test_anonymous_returns_200(
        self, mock_query_get_by_id, mock_oauth2_post_request
    ):
        """test_anonymous_returns_200"""

        response = self.send_oauth_request(
            mock_query_get_by_id, mock_oauth2_post_request, None
        )
        self.assertEqual(response.status_code, status.HTTP_200_OK)

    @patch.object(pid_views, "oauth2_post_request")
    @patch.object(query_api, "get_by_id")
    def test_authenticated_returns_200(
        self, mock_query_get_by_id, mock_oauth2_post_request
    ):
        """test_authenticated_returns_200"""

        mock_user = create_mock_user("1")

        response = self.send_oauth_request(
            mock_query_get_by_id, mock_oauth2_post_request, mock_user
        )
        self.assertEqual(response.status_code, status.HTTP_200_OK)

    @patch.object(pid_views, "oauth2_post_request")
    @patch.object(query_api, "get_by_id")
    def test_staff_returns_200(
        self, mock_query_get_by_id, mock_oauth2_post_request
    ):
        """test_staff_returns_200"""

        mock_user = create_mock_user("1", is_staff=True)

        response = self.send_oauth_request(
            mock_query_get_by_id, mock_oauth2_post_request, mock_user
        )
        self.assertEqual(response.status_code, status.HTTP_200_OK)


class TestTemplateHtmlRenderingByDataPIDPermission(SimpleTestCase):
    """TestTemplateHtmlRenderingByDataPIDPermission"""

    def test_anonymous_returns_http_403(self):
        """test_anonymous_returns_http_403

        Returns:

        """
        response = RequestMock.do_request_get(
            pid_views.DataHtmlRenderByPID.as_view(), None
        )

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    @patch.object(data_api, "get_data_by_pid")
    @patch.object(template_html_rendering_api, "get_by_template_id")
    @patch.object(template_html_rendering_api, "render_data")
    def test_authenticated_returns_http_200(
        self,
        mock_data_api_get_data_by_pid,
        mock_template_html_rendering_api_get_by_template_id,
        mock_template_html_rendering_api_render_data,
    ):
        """test_authenticated_returns_http_200

         Args:
            mock_data_api_get_data_by_pid:
            mock_template_html_rendering_api_get_by_template_id:
            mock_template_html_rendering_api_render_data:

        Returns:

        """
        mock_data_api_get_data_by_pid.return_value = Mock()
        mock_template_html_rendering_api_get_by_template_id.return_value = (
            Mock()
        )
        mock_template_html_rendering_api_render_data.return_value = Mock()
        mock_user = create_mock_user("1")

        response = RequestMock.do_request_get(
            pid_views.DataHtmlRenderByPID.as_view(),
            mock_user,
            data={"pid": ""},
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)

    @patch.object(data_api, "get_data_by_pid")
    @patch.object(template_html_rendering_api, "get_by_template_id")
    @patch.object(template_html_rendering_api, "render_data")
    def test_staff_returns_http_200(
        self,
        mock_data_api_get_data_by_pid,
        mock_template_html_rendering_api_get_by_template_id,
        mock_template_html_rendering_api_render_data,
    ):
        """test_staff_returns_http_200

         Args:
            mock_data_api_get_data_by_pid:
            mock_template_html_rendering_api_get_by_template_id:
            mock_template_html_rendering_api_render_data:

        Returns:

        """
        mock_data_api_get_data_by_pid.return_value = Mock()
        mock_template_html_rendering_api_get_by_template_id.return_value = (
            Mock()
        )
        mock_template_html_rendering_api_render_data.return_value = Mock()
        mock_user = create_mock_user("1", is_staff=True)

        response = RequestMock.do_request_get(
            pid_views.DataHtmlRenderByPID.as_view(),
            mock_user,
            data={"pid": ""},
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)



""" Utilities to retrieve path information.
"""


def get_api_path_from_object(instance):
    """Retrieve API path from a given object instance.

    Code adapted from https://stackoverflow.com/questions/2020014.

    Args:
        instance:

    Returns:
        str: A dot delimited path to the API of the given object
    """
    instance_class = instance.__class__
    module = instance_class.__module__

    if module == "__builtin__":
        # avoid outputs like '__builtin__.str'
        model_path = instance_class.__name__
    else:
        model_path = module + "." + instance_class.__name__

    return f"{'.'.join(model_path.split('.')[:-2])}.api"



""" Utilities related to blobs with PID
"""

import logging
import re

from django.urls import reverse

from core_linked_records_app.system.blob import api as blob_system_api
from core_linked_records_app.system.local_id import api as local_id_system_api

logger = logging.getLogger(__name__)


def get_blob_download_regex(xml_string):
    """Retrieve a list of blob PID from an XML document. Performs a DB lookup
    to ensure the PID belong to blobs.

    Args:
        xml_string (str): Content of the XML file

    Returns:
        list<str>: List of blobs found in the given text
    """
    mock_string = "mock_string"

    # Build django url to download a blob using PID
    blob_pid_url = reverse(
        "core_linked_records_provider_record",
        kwargs={"provider": mock_string, "record": mock_string},
    )
    blob_pid_url = re.sub(f"{mock_string}/?", "", blob_pid_url)

    # Apply the regex
    document_pid_list = re.findall(
        f">(http[s]?:[^<>]+{blob_pid_url}[^/]+/[^/]+/[^/]+/?)<", xml_string
    )

    blob_url_list = []

    for document_pid in document_pid_list:
        try:
            record_name = "/".join(document_pid.split("/")[-2:])
            record_object = local_id_system_api.get_by_name(record_name)
            blob_system_api.get_pid_for_blob(record_object.record_object_id)
            blob_url_list.append(document_pid)
        except Exception as exc:  # pylint: disable=broad-except
            logger.warning(
                "Retrieving blob URL raised an exception: %s", str(exc)
            )
            continue

    return blob_url_list



""" Permission tests for core_linked_records_app.rest.blob.views
"""

from unittest import TestCase
from unittest.mock import patch

from rest_framework import status

from core_linked_records_app.components.blob import api as blob_api
from core_linked_records_app.rest.blob import views as blob_views
from core_linked_records_app.system.local_id import api as local_id_system_api
from core_main_app.commons.exceptions import DoesNotExist
from core_main_app.rest.blob.views import BlobList
from core_main_app.utils.tests_tools.MockUser import create_mock_user
from core_main_app.utils.tests_tools.RequestMock import RequestMock
from tests import mocks


class TestBlobUploadWithPIDViewPost(TestCase):
    """Test Blob Upload With PID View Post"""

    def test_anonymous_returns_403(self):
        """test_anonymous_returns_403"""

        response = RequestMock.do_request_post(
            blob_views.BlobUploadWithPIDView.as_view(), None
        )

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    def test_authenticated_returns_403(self):
        """test_authenticated_returns_403"""

        mock_user = create_mock_user("1")

        response = RequestMock.do_request_post(
            blob_views.BlobUploadWithPIDView.as_view(), mock_user
        )

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    def test_staff_returns_403(self):
        """test_staff_returns_403"""

        mock_user = create_mock_user("1", is_staff=True)

        response = RequestMock.do_request_post(
            blob_views.BlobUploadWithPIDView.as_view(), mock_user
        )

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    @patch.object(blob_api, "set_pid_for_blob")
    @patch.object(BlobList, "post")
    @patch.object(local_id_system_api, "get_by_name")
    def test_superuser_returns_201(
        self, mock_get_by_name, mock_blob_list_post, mock_set_pid_for_blob
    ):
        """test_superuser_returns_201"""

        mock_user = create_mock_user("1", is_staff=True, is_superuser=True)

        mock_get_by_name.side_effect = DoesNotExist(
            "mock_get_by_name_exception"
        )
        mock_blob_list_post.return_value = mocks.MockResponse(
            data={"id": "mock_id"}, status_code=status.HTTP_201_CREATED
        )
        mock_set_pid_for_blob.return_value = None

        response = RequestMock.do_request_post(
            blob_views.BlobUploadWithPIDView.as_view(),
            mock_user,
            data={"pid": "mock_pid"},
            content_type=None,
        )

        self.assertEqual(response.status_code, status.HTTP_201_CREATED)



""" Unit tests for core_linked_records_app.components.pid_path.models
"""

from unittest import TestCase
from unittest.mock import patch, MagicMock

from django.core.exceptions import ObjectDoesNotExist

from core_linked_records_app.components.pid_path.models import PidPath
from core_main_app.commons.exceptions import ModelError
from core_main_app.components.template.models import Template


class TestPidPathGetAll(TestCase):
    """Unit tests for `PidPath.get_all` method."""

    @patch.object(PidPath, "objects")
    def test_pid_path_all_failure_raises_model_error(self, mock_pid_path):
        """test_pid_path_all_failure_raises_model_error"""

        mock_pid_path.all.side_effect = Exception(
            "mock_pid_path_all_exception"
        )

        with self.assertRaises(ModelError):
            PidPath.get_all()

    @patch.object(PidPath, "objects")
    def test_returns_pid_path_all_output(self, mock_pid_path):
        """test_returns_pid_path_all_output"""

        expected_result = "mock_pid_path_all"
        mock_pid_path.all.return_value = expected_result

        self.assertEqual(PidPath.get_all(), expected_result)


class TestPidPathGetByTemplateList(TestCase):
    """Unit tests for `PidPath.get_all_by_template_list` method."""

    @patch.object(PidPath, "objects")
    def test_pid_path_filter_failure_raises_model_error(self, mock_pid_path):
        """test_pid_path_filter_failure_raises_model_error"""

        mock_pid_path.filter.side_effect = Exception(
            "mock_pid_path_filter_exception"
        )

        with self.assertRaises(ModelError):
            PidPath.get_all_by_template_list("mock_template_list")

    @patch.object(PidPath, "objects")
    def test_returns_pid_path_filter_output(self, mock_pid_path):
        """test_returns_pid_path_filter_output"""

        expected_result = "mock_pid_path_filter"
        mock_pid_path.filter.return_value = expected_result

        self.assertEqual(
            PidPath.get_all_by_template_list("mock_template_list"),
            expected_result,
        )


class TestPidPathGetByTemplate(TestCase):
    """Unit tests for `PidPath.get_by_template` method."""

    @patch.object(PidPath, "objects")
    def test_pid_path_get_does_not_exists_returns_none(self, mock_pid_path):
        """test_pid_path_get_does_not_exists_returns_none"""

        mock_pid_path.get.side_effect = ObjectDoesNotExist()

        self.assertIsNone(PidPath.get_by_template("mock_template"))

    @patch.object(PidPath, "objects")
    def test_pid_path_get_raises_model_error(self, mock_pid_path):
        """test_pid_path_get_raises_model_error"""

        mock_pid_path.get.side_effect = Exception(
            "mock_pid_path_get_exception"
        )

        with self.assertRaises(ModelError):
            PidPath.get_by_template("mock_template")

    @patch.object(PidPath, "objects")
    def test_returns_pid_path_get_output(self, mock_pid_path):
        """test_returns_pid_path_get_output"""

        expected_result = "mock_pid_path_get"
        mock_pid_path.get.return_value = expected_result

        self.assertEqual(
            PidPath.get_by_template("mock_template"), expected_result
        )


class TestPidPathStr(TestCase):
    """Unit tests for `PidPath.__str__` method."""

    def test_str_is_correct(self):
        mock_path = MagicMock()
        mock_template = Template()
        pid_path_object = PidPath()
        pid_path_object.path = mock_path
        pid_path_object.template = mock_template

        self.assertEqual(
            pid_path_object.__str__(),
            f"PID path '{mock_path}' for template '{mock_template}'",
        )



""" Menu configuration for core_linked_records_app.
Upon installation of the app the following menus are displayed:

  * Admin menu

    * PID settings
"""

from django.urls import reverse
from menu import Menu, MenuItem

pid_settings_menu = MenuItem(
    "PID Settings",
    reverse("core-admin:core_linked_records_app_admin_settings"),
    icon="cogs",
)

admin_menu = MenuItem("LINKED RECORDS", None, children=(pid_settings_menu,))

Menu.add_item("admin", admin_menu)



""" Apps file for setting linked records when app is ready
"""

import sys

from django.apps import AppConfig


class LinkedRecordsAppConfig(AppConfig):
    """Core application settings"""

    name = "core_linked_records_app"
    verbose_name = "Core Linked Records App"

    def ready(self):
        """Run when the app is ready.

        Returns:

        """
        from core_main_app.commons.exceptions import CoreError
        from core_linked_records_app import settings
        from core_linked_records_app.access_control import (
            discover as acl_discover,
        )
        from core_linked_records_app.components.blob import watch as blob_watch
        from core_linked_records_app.components.data import watch as data_watch
        from core_linked_records_app.components.pid_settings import (
            watch as pid_settings_watch,
        )

        if "" in settings.ID_PROVIDER_PREFIXES:
            raise CoreError(
                "Empty string not allowed in settings.ID_PROVIDER_PREFIXES."
            )

        if "migrate" not in sys.argv:
            acl_discover.init_permissions()
            pid_settings_watch.init()
            data_watch.init()
            blob_watch.init()



""" Linked records REST views
"""

import json
import logging

from rest_framework import status
from rest_framework.parsers import JSONParser
from rest_framework.renderers import JSONRenderer
from rest_framework.response import Response
from rest_framework.views import APIView

from core_linked_records_app.components.blob.api import get_blob_by_pid
from core_linked_records_app.components.data.api import get_data_by_pid
from core_linked_records_app.rest.data.renderers.data_html_user_renderer import (
    DataHtmlUserRenderer,
)
from core_linked_records_app.rest.data.renderers.data_xml_renderer import (
    DataXmlRenderer,
)
from core_linked_records_app.utils.exceptions import (
    InvalidPrefixError,
    InvalidRecordError,
)
from core_linked_records_app.utils.pid import split_prefix_from_record
from core_linked_records_app.utils.providers import ProviderManager
from core_main_app.access_control.exceptions import AccessControlError
from core_main_app.commons.exceptions import DoesNotExist
from core_main_app.rest.data.serializers import DataSerializer
from core_main_app.utils.file import get_file_http_response

logger = logging.getLogger(__name__)


class ProviderRecordView(APIView):
    """Provider Record View"""

    parser_classes = (JSONParser,)
    renderer_classes = (DataHtmlUserRenderer, JSONRenderer, DataXmlRenderer)

    def __init__(self, **kwargs):
        self.provider_manager = ProviderManager()
        super().__init__(**kwargs)

    def post(self, request, provider, record):
        """Create a handle record

        Args:
            request:
            provider:
            record:

        Returns:
        """
        try:
            try:
                prefix, record = split_prefix_from_record(record)
            except InvalidPrefixError as invalid_prefix_error:
                return Response(
                    {
                        "record": record,
                        "url": request.build_absolute_uri("?"),
                        "message": str(invalid_prefix_error),
                    },
                    status=status.HTTP_500_INTERNAL_SERVER_ERROR,
                )
            except InvalidRecordError as invalid_record_error:
                return Response(
                    {
                        "record": record,
                        "url": request.build_absolute_uri("?"),
                        "message": str(invalid_record_error),
                    },
                    status=status.HTTP_500_INTERNAL_SERVER_ERROR,
                )

            id_provider = self.provider_manager.get(provider)
            provider_response = id_provider.create(prefix, record)

            provider_content = json.loads(provider_response.content)
            return Response(
                provider_content, status=provider_response.status_code
            )
        except Exception as exc:  # pylint: disable=broad-except
            return Response(
                {
                    "record": record,
                    "url": request.build_absolute_uri("?"),
                    "message": str(exc),
                },
                status=status.HTTP_500_INTERNAL_SERVER_ERROR,
            )

    def put(
        self,
        request,  # noqa, pylint: disable=unused-argument
        provider,
        record,
    ):
        """Update the value of a given handle record

        Args:
            request:
            provider:
            record:

        Returns:
        """
        try:
            id_provider = self.provider_manager.get(provider)
            provider_response = id_provider.update(record)

            provider_content = json.loads(provider_response.content)

            return Response(
                provider_content, status=provider_response.status_code
            )
        except Exception as exc:  # pylint: disable=broad-except
            return Response(
                {
                    "message": f"An unexpected error occurred while updating "
                    f"record: {str(exc)}"
                },
                status=status.HTTP_500_INTERNAL_SERVER_ERROR,
            )

    def get(self, request, provider, record):
        """Retrieve the local data of a given handle record

        Args:
            request:
            provider:
            record:

        Returns:
        """
        try:
            id_provider = self.provider_manager.get(provider)
            provider_response = id_provider.get(record)

            try:
                query_result = get_data_by_pid(
                    json.loads(provider_response.content)["url"], request
                )
                return Response(
                    DataSerializer(query_result).data,
                    status=status.HTTP_200_OK,
                )
            except DoesNotExist:
                # Try to find PID in blobs
                try:
                    query_result = get_blob_by_pid(
                        json.loads(provider_response.content)["url"],
                        request.user,
                    )

                    return get_file_http_response(
                        query_result.blob.read(), query_result.filename
                    )
                except AccessControlError as exception:
                    content = {"message": str(exception)}
                    return Response(content, status=status.HTTP_403_FORBIDDEN)
                except DoesNotExist:
                    content = {
                        "status": "error",
                        "code": status.HTTP_404_NOT_FOUND,
                        "message": "No document with specified handle found",
                    }
                    return Response(content, status=status.HTTP_404_NOT_FOUND)
        except Exception as exc:  # pylint: disable=broad-except
            content = {
                "status": "error",
                "code": status.HTTP_500_INTERNAL_SERVER_ERROR,
                "message": str(exc),
            }
            return Response(
                content, status=status.HTTP_500_INTERNAL_SERVER_ERROR
            )

    def delete(
        self,
        request,  # noqa, pylint: disable=unused-argument
        provider,
        record,
    ):
        """Delete a handle record

        Args:
            request:
            provider:
            record:

        Returns:
        """
        try:
            id_provider = self.provider_manager.get(provider)
            provider_response = id_provider.delete(record)

            provider_content = json.loads(provider_response.content)

            return Response(
                provider_content, status=provider_response.status_code
            )
        except Exception as exc:  # pylint: disable=broad-except
            return Response(
                {
                    "message": f"An unexpected error occurred while deleting "
                    f"record: {str(exc)}"
                },
                status=status.HTTP_500_INTERNAL_SERVER_ERROR,
            )



""" Unit tests for core_linked_records_app.rest.blob.views
"""

from unittest import TestCase
from unittest.mock import patch

from rest_framework import status

from core_linked_records_app.components.blob import api as blob_api
from core_linked_records_app.rest.blob import views as blob_views
from core_linked_records_app.system.local_id import api as local_id_system_api
from core_main_app.commons.exceptions import DoesNotExist
from core_main_app.rest.blob.views import BlobList
from core_main_app.utils.tests_tools.MockUser import create_mock_user
from tests import mocks


class TestBlobUploadWithPIDViewPost(TestCase):
    """Test Blob Upload With PID View Post"""

    def setUp(self) -> None:
        self.mock_request = mocks.MockRequest()
        self.mock_request.user = create_mock_user("1", is_superuser=True)
        self.mock_request.POST = {}

    def test_pid_none_returns_500(self):
        """test_pid_none_returns_500"""

        test_view = blob_views.BlobUploadWithPIDView()
        response = test_view.post(self.mock_request)

        self.assertEqual(response.status_code, 500)

    @patch.object(local_id_system_api, "get_by_name")
    def test_local_pid_get_by_name_fails_returns_500(self, mock_get_by_name):
        """test_local_pid_get_by_name_fails_returns_500"""

        self.mock_request.POST["pid"] = "mock_pid"
        mock_get_by_name.side_effect = Exception("mock_get_by_name_exception")

        test_view = blob_views.BlobUploadWithPIDView()
        response = test_view.post(self.mock_request)

        self.assertEqual(response.status_code, 500)

    @patch.object(local_id_system_api, "get_by_name")
    def test_local_pid_get_by_name_succeeds_returns_500(
        self, mock_get_by_name
    ):
        """test_local_pid_get_by_name_succeeds_returns_500"""

        self.mock_request.POST["pid"] = "mock_pid"
        mock_get_by_name.return_value = None

        test_view = blob_views.BlobUploadWithPIDView()
        response = test_view.post(self.mock_request)

        self.assertEqual(response.status_code, 500)

    @patch.object(BlobList, "post")
    @patch.object(local_id_system_api, "get_by_name")
    def test_super_post_fails_returns_500(
        self, mock_get_by_name, mock_blob_list_post
    ):
        """test_super_post_fails_returns_500"""

        self.mock_request.POST["pid"] = "mock_pid"
        mock_get_by_name.side_effect = DoesNotExist(
            "mock_get_by_name_does_not_exist"
        )
        mock_blob_list_post.side_effect = Exception(
            "mock_blob_list_post_exception"
        )

        test_view = blob_views.BlobUploadWithPIDView()
        response = test_view.post(self.mock_request)

        self.assertEqual(response.status_code, 500)

    @patch.object(BlobList, "post")
    @patch.object(local_id_system_api, "get_by_name")
    def test_super_post_returns_400_returns_400(
        self, mock_get_by_name, mock_blob_list_post
    ):
        """test_super_post_returns_400_returns_400"""

        self.mock_request.POST["pid"] = "mock_pid"
        mock_get_by_name.side_effect = DoesNotExist(
            "mock_get_by_name_does_not_exist"
        )
        mock_blob_list_post.return_value = mocks.MockResponse(
            status_code=status.HTTP_400_BAD_REQUEST
        )

        test_view = blob_views.BlobUploadWithPIDView()
        response = test_view.post(self.mock_request)

        self.assertEqual(response.status_code, 400)

    @patch.object(blob_api, "set_pid_for_blob")
    @patch.object(BlobList, "post")
    @patch.object(local_id_system_api, "get_by_name")
    def test_set_pid_for_blob_fails_returns_500(
        self, mock_get_by_name, mock_blob_list_post, mock_set_pid_for_blob
    ):
        """test_set_pid_for_blob_fails_returns_500"""

        self.mock_request.POST["pid"] = "mock_pid"
        mock_get_by_name.side_effect = DoesNotExist(
            "mock_get_by_name_does_not_exist"
        )
        mock_blob_list_post.return_value = mocks.MockResponse(
            status_code=status.HTTP_201_CREATED, data={"id": "mock_blob_id"}
        )
        mock_set_pid_for_blob.side_effect = Exception(
            "mock_set_pid_for_blob_exception"
        )

        test_view = blob_views.BlobUploadWithPIDView()
        response = test_view.post(self.mock_request)

        self.assertEqual(response.status_code, 500)

    @patch.object(blob_api, "set_pid_for_blob")
    @patch.object(BlobList, "post")
    @patch.object(local_id_system_api, "get_by_name")
    def test_success_returns_201(
        self, mock_get_by_name, mock_blob_list_post, mock_set_pid_for_blob
    ):
        """test_success_returns_201"""

        self.mock_request.POST["pid"] = "mock_pid"
        mock_get_by_name.side_effect = DoesNotExist(
            "mock_get_by_name_does_not_exist"
        )
        mock_blob_list_post.return_value = mocks.MockResponse(
            status_code=status.HTTP_201_CREATED, data={"id": "mock_blob_id"}
        )
        mock_set_pid_for_blob.return_value = None

        test_view = blob_views.BlobUploadWithPIDView()
        response = test_view.post(self.mock_request)

        self.assertEqual(response.status_code, 201)



""" Unit tests for `core_linked_records.components.pid_path.access_control`.
"""

from unittest import TestCase
from unittest.mock import patch, MagicMock

from core_linked_records_app.components.pid_path import (
    access_control as pid_path_acl,
)
from core_main_app.access_control.exceptions import AccessControlError
from core_main_app.utils.tests_tools.MockUser import create_mock_user


class TestCanGetByTemplate(TestCase):
    """Unit tests for `can_get_by_template` function."""

    def setUp(self) -> None:
        """setUp"""
        self.mock_kwargs = {
            "func": MagicMock(),
            "template": MagicMock(),
            "user": create_mock_user("1"),
        }

    @patch.object(pid_path_acl, "check_can_read_template")
    def test_check_can_read_template_called(
        self, mock_check_can_read_template
    ):
        """test_check_can_read_template_called"""
        pid_path_acl.can_get_by_template(**self.mock_kwargs)

        mock_check_can_read_template.assert_called_with(
            self.mock_kwargs["template"], self.mock_kwargs["user"]
        )

    @patch.object(pid_path_acl, "check_can_read_template")
    def test_check_can_read_template_failure_raises_acl_error(
        self, mock_check_can_read_template
    ):
        """test_check_can_read_template_failure_raises_acl_error"""
        mock_check_can_read_template.side_effect = AccessControlError(
            "mock_check_can_read_template_acl_error"
        )

        with self.assertRaises(AccessControlError):
            pid_path_acl.can_get_by_template(**self.mock_kwargs)

    @patch.object(pid_path_acl, "check_can_read_template")
    def test_successful_execution_returns_func_with_args(
        self,
        mock_check_can_read_template,  # noqa, pylint: disable=unused-argument
    ):
        """test_successful_execution_returns_func_with_args"""
        self.assertEqual(
            pid_path_acl.can_get_by_template(**self.mock_kwargs),
            self.mock_kwargs["func"](
                self.mock_kwargs["template"], self.mock_kwargs["user"]
            ),
        )



""" Permission tests for core_linked_records_app.rest.pid_path.views
"""

from unittest import TestCase
from unittest.mock import patch

from rest_framework import status
from rest_framework.generics import (
    ListCreateAPIView,
    RetrieveUpdateDestroyAPIView,
)
from rest_framework.response import Response

from core_linked_records_app.rest.pid_path import views as pid_path_views
from core_main_app.utils.tests_tools.MockUser import create_mock_user
from core_main_app.utils.tests_tools.RequestMock import RequestMock


class TestPidPathListViewGet(TestCase):
    """Unit tests for `PidPathListView.get` method."""

    def test_anonymous_returns_403(self):
        """test_anonymous_returns_403"""

        response = RequestMock.do_request_get(
            pid_path_views.PidPathListView.as_view(), None
        )

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    def test_authenticated_returns_200(self):
        """test_authenticated_returns_200"""

        mock_user = create_mock_user("1")

        response = RequestMock.do_request_get(
            pid_path_views.PidPathListView.as_view(), mock_user
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)

    def test_staff_returns_200(self):
        """test_staff_returns_200"""

        mock_user = create_mock_user("1", is_staff=True)

        response = RequestMock.do_request_get(
            pid_path_views.PidPathListView.as_view(), mock_user
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)


class TestPidPathListViewPost(TestCase):
    """Unit tests for `PidPathListView.post` method."""

    def test_anonymous_returns_403(self):
        """test_anonymous_returns_403"""

        response = RequestMock.do_request_post(
            pid_path_views.PidPathListView.as_view(),
            None,
            data={"path": "mock.path", "template": "mock_template_id"},
        )

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    @patch.object(ListCreateAPIView, "post")
    def test_authenticated_returns_201(self, mock_view_post):
        """test_authenticated_returns_201"""

        mock_user = create_mock_user("1")

        mock_view_post.return_value = Response(status=status.HTTP_201_CREATED)
        response = RequestMock.do_request_post(
            pid_path_views.PidPathListView.as_view(),
            mock_user,
            data={"path": "mock.path", "template": "mock_template_id"},
        )

        self.assertEqual(response.status_code, status.HTTP_201_CREATED)

    @patch.object(ListCreateAPIView, "post")
    def test_staff_returns_201(self, mock_view_post):
        """test_staff_returns_201"""

        mock_user = create_mock_user("1", is_staff=True)

        mock_view_post.return_value = Response(status=status.HTTP_201_CREATED)
        response = RequestMock.do_request_post(
            pid_path_views.PidPathListView.as_view(),
            mock_user,
            data={"path": "mock.path", "template": "mock_template_id"},
        )

        self.assertEqual(response.status_code, status.HTTP_201_CREATED)


class TestPidPathDetailViewGet(TestCase):
    """Unit tests for `PidPathDetailView.get` method."""

    def test_anonymous_returns_403(self):
        """test_anonymous_returns_403"""

        response = RequestMock.do_request_get(
            pid_path_views.PidPathDetailView.as_view(), None, param={"id": 0}
        )

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    @patch.object(RetrieveUpdateDestroyAPIView, "get")
    def test_authenticated_returns_200(self, mock_view_get):
        """test_authenticated_returns_200"""

        mock_user = create_mock_user("1")

        mock_view_get.return_value = Response(status=status.HTTP_200_OK)
        response = RequestMock.do_request_get(
            pid_path_views.PidPathDetailView.as_view(),
            mock_user,
            param={"id": 0},
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)

    @patch.object(RetrieveUpdateDestroyAPIView, "get")
    def test_staff_returns_200(self, mock_view_get):
        """test_staff_returns_200"""

        mock_user = create_mock_user("1", is_staff=True)

        mock_view_get.return_value = Response(status=status.HTTP_200_OK)
        response = RequestMock.do_request_get(
            pid_path_views.PidPathDetailView.as_view(),
            mock_user,
            param={"id": 0},
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)


class TestPidPathDetailViewPatch(TestCase):
    """Unit tests for `PidPathDetailView.patch` method."""

    def test_anonymous_returns_403(self):
        """test_anonymous_returns_403"""

        response = RequestMock.do_request_patch(
            pid_path_views.PidPathDetailView.as_view(),
            None,
            data={"path": "mock.path", "template": "mock_template_id"},
        )

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    @patch.object(RetrieveUpdateDestroyAPIView, "patch")
    def test_authenticated_returns_200(self, mock_view_patch):
        """test_authenticated_returns_200"""

        mock_user = create_mock_user("1")

        mock_view_patch.return_value = Response(status=status.HTTP_200_OK)
        response = RequestMock.do_request_patch(
            pid_path_views.PidPathDetailView.as_view(),
            mock_user,
            data={"path": "mock.path", "template": "mock_template_id"},
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)

    @patch.object(RetrieveUpdateDestroyAPIView, "patch")
    def test_staff_returns_200(self, mock_view_patch):
        """test_staff_returns_200"""

        mock_user = create_mock_user("1", is_staff=True)

        mock_view_patch.return_value = Response(status=status.HTTP_200_OK)
        response = RequestMock.do_request_patch(
            pid_path_views.PidPathDetailView.as_view(),
            mock_user,
            data={"path": "mock.path", "template": "mock_template_id"},
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)


class TestPidPathDetailViewDelete(TestCase):
    """Unit tests for `PidPathDetailView.delete` method."""

    def test_anonymous_returns_403(self):
        """test_anonymous_returns_403"""

        response = RequestMock.do_request_delete(
            pid_path_views.PidPathDetailView.as_view(),
            None,
        )

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    @patch.object(RetrieveUpdateDestroyAPIView, "delete")
    def test_authenticated_returns_204(self, mock_view_delete):
        """test_authenticated_returns_204"""

        mock_user = create_mock_user("1")

        mock_view_delete.return_value = Response(
            status=status.HTTP_204_NO_CONTENT
        )
        response = RequestMock.do_request_delete(
            pid_path_views.PidPathDetailView.as_view(),
            mock_user,
        )

        self.assertEqual(response.status_code, status.HTTP_204_NO_CONTENT)

    @patch.object(RetrieveUpdateDestroyAPIView, "delete")
    def test_staff_returns_204(self, mock_view_delete):
        """test_staff_returns_204"""

        mock_user = create_mock_user("1", is_staff=True)

        mock_view_delete.return_value = Response(
            status=status.HTTP_204_NO_CONTENT
        )
        response = RequestMock.do_request_delete(
            pid_path_views.PidPathDetailView.as_view(),
            mock_user,
        )

        self.assertEqual(response.status_code, status.HTTP_204_NO_CONTENT)
